import platform

def Copy(text):
 """Copies text to the clipboard."""

 plat = platform.system()
 if plat == 'Windows':
  import win32clipboard
  import win32con
  win32clipboard.OpenClipboard()
  win32clipboard.EmptyClipboard()
  win32clipboard.SetClipboardText(text, win32con.CF_UNICODETEXT)
  win32clipboard.CloseClipboard()
 elif plat == 'Linux':
  import gtk
  cb = gtk.Clipboard()
  cb.set_text(text)
  cb.store()
 elif plat == 'Darwin':
  import Carbon.Scrap
  Carbon.Scrap.ClearCurrentScrap()
  scrap = Carbon.Scrap.GetCurrentScrap()
  scrap.PutScrapFlavor('TEXT', 0, text)
 return True



#This module deals with sound and music playback
from random import choice
from pygame.mixer import *
from pygame.mixer import Sound as Mixer_sound
pre_init(44100,-16, 2, 2048)
init()
set_num_channels(50)

# event queue for scheduling end events
from pyaudiogame import event_queue

# Position is to controll the 3D audio
from pyaudiogame import position

# this is a list of currently playing sounds so that when the listener moves, the listener can be updated on any looping sound
playing_sounds = []

# The listener is set facing north at 90 degrees. east is 0 degrees and the positions move counterclockwise.
position.set_listener(0,0,90)

def get_listener():
	return position.get_listener()

def set_listener(x,y,o):
	"""Sets pos of the listener. east is o=0 and north is o=90"""
	position.set_listener(x,y,o)
	[sound.set_volume() for sound in playing_sounds]

class Sound(object):
	"""
		todo is to wrap all the channel methods so they only work on the sound. This could happen either by checking if the sound is playing on the channel or possibly by calling methods on the sound itself.
		mixer documentation:
		https://www.pygame.org/docs/ref/mixer.html

		This class ads support for 3D positioning. Just pass the position of the sound in a tuple of (x,y) coordinates after the path of the file.
	"""

	def __init__(self, filename, position=None, single_channel=True):
		self.name = filename
		self.pos = position # should be a tuple (x, y)
		self.channel = None
		self.paused = False
		self.playing = False
		self.stopped = False # this is to tell a callback if the sound was stopped by a stop function or not.
		self.single_channel = single_channel
		self.callback = lambda e: None # the callback is passed this object as an argument and is triggered at the end of the sound
		self._id = "sound-%s-%s" % (self.name, id(self))

		try:
			self.sound = Mixer_sound(filename)
		except:
			raise Exception("Unable to open file %r" % filename)


	def play(self, loops=0, maxtime=0, fade_ms=0):
		self.playing = True
		self.stopped = False
		if not self.channel or ((not self.single_channel or self.channel.get_sound() != self.sound) and self.channel.get_busy()):
			self.channel = find_channel() or self.channel
		self.channel.play(self.sound, loops, maxtime, fade_ms)
		if self.pos:
			playing_sounds.append(self)
			self.set_volume()
		event_queue.schedule(function=self.check_if_finished, repeats=-1, delay=0, name=self._id) # this uses the channel.get_busy to figure out if the sound has finished playing.
#		event_queue.schedule(function=self.finish, repeats=1, delay=self.get_length()-0.09, name=self._id) # This does the same as above, but uses the length of the sound to schedule an end event. The problem with this is that if one pauses the sound, the event still runs. The pro is that the end event can be faster than the actual sound.

	def get_volume(self):
		return self.channel.get_volume()

	def move_pos(self, x=0, y=0):
		cx, cy = self.pos
		self.set_pos(cx+x, cy+y)
		return self.pos

	def set_pos(self, x, y):
		self.pos = (float(x), float(y))
		if(self.channel):
			self.channel.set_volume(*position.stereo(*self.pos))

	def get_pos(self):
		return self.pos

	def get_length(self):
		return self.sound.get_length()

	def toggle_pause(self):
		"""This function can be called to pause and unpause a sound without the script needing to handle the check for paused and unpaused. If the sound is paused when this function is called, then the sound is unpaused and if the sound is playing when this function is called, then the sound is paused. It's very good for buttons to play and pause sounds."""
		if not self.channel or not self.playing:
			self.play()
		elif self.paused:
			self.unpause()
		else:
			self.pause()

	def unpause(self):
		if not self.channel: return False
		self.channel.unpause()
		self.paused = False

	def is_paused(self):
		return self.paused

	def pause(self):
		if not self.channel: return False
		self.channel.pause()
		self.paused = True

	def stop(self):
		"""This stops the channel from playing."""
		event_queue.unschedule(self._id)
		if self in playing_sounds:
			playing_sounds.remove(self)
		self.playing = False
		self.paused = False
		self.stopped = True
		self.unpause()
		if self.channel:
			self.channel.stop()
		self.finish()

	def stop_sound(self):
		"""This stops the sound object from playing rather than the channel"""
		self.sound.stop()
		self.unpaws()
		self.playing = False
		mixer_queue.remove(self.channel)
		if self in playing_sounds:
			playing_sounds.remove(self)

	def get_busy(self):
		"""Returns if the channel is active. This is used for triggering the callback"""
		if self.channel:
			return self.channel.get_busy()
		return False

	def check_if_finished(self):
		"""This runs every tick to see if the channel is done. if it is done, then it runs the finish method and removes itself from the event_queue."""
		if not self.get_busy():
			self.finish()
			event_queue.unschedule(self._id)

	def toggle_playing(self):
		if self.playing:
			self.stop()
		else:
			self.play()

	def set_volume(self, volume=1, right=None):
		"""Sets the volume if there is a channel. If there is a position, then volume is adjusted to be that position. If no arguments are passed, then it will update the volume to be the current pos, or set volume back to 1."""
		if not  self.channel:
			return 0
		if volume > 1:
			volume = 1
		elif volume <= 0:
			self.channel.set_volume(0)
			return 0
		if not self.pos and not right:
			self.channel.set_volume(volume)
		elif right:
			self.channel.set_volume(volume, right)
		else:
			x, y = self.pos
			if x == 0 and y == 0:
				self.channel.set_volume(volume)
				return 0
			self.channel.set_volume(*position.stereo(x/volume, y/volume))

	def finish(self):
		self.playing = False
		self.paused = False
		self.callback(self)

mixer_paused = False
def mixer_toggle_pause(excluded_channels=[]):
	global mixer_paused
	if mixer_paused:
		unpause()
		[c.pause() for c in excluded_channels if c.is_paused()]
		mixer_paused = False
	else:
		pause()
		mixer_paused = True
		[c.unpause() for c in excluded_channels if not c.is_paused()]

class SoundIterator(object):
	def __init__(self, *args, random=False, random_repeat=True):
		"""Pass in sound files, or SoundIterator objects (anything with a play method and an on_end event (if the play_loop functionality is desired)), and if random is false, then they will be played in order. Otherwise they will be played randomly. Sound objects can be passed in as well, but their callback functionality will be replaced. It is better to pass in strings."""
		self.sounds = []
		self.random = random
		self.random_repeat = random_repeat
		self.index = 0
		self.event_name = "%sEndSoundEvent" % id(self)
		self.playing = False
		for s in args:
			if type(s) == str:
				s = Sound(s)
			self.sounds.append(s)

	def play(self, looping=False):
		"""Plays sounds"""
		self.playing = True
		if looping:
			self._play_looping()
		else:
			self._play_single()

	def toggle_looping(self, current_sound=True):
		if self.playing:
			self.stop(current_sound)
		else:
			self.play(looping=True)

	def _play_looping(self, e=None):
		"""plays the sounds until stop is called. makes this function the callback of the sound. If this sound is called, it should not have a callback as it overwrites the callback"""
		if self.playing:
			s = self.sounds[self.index]
			s.callback = self._play_looping
			self._play_single()

	def stop(self, current_sound=True):
		"""Stops all sounds. If current_sound is False, then the current sound will finish, and no more sounds will play."""
		self.playing = False
		if current_sound:
			[s.stop() for s in self.sounds]

	def _play_single(self):
		"""Will play either the next sound, or a random sound. If there are no sounds, it plays no sounds."""
		if not self.sounds:
			return False
		self.sounds[self.index].play()
		if self.random:
			if self.random_repeat:
				self.index = choice(list(range(len(self.sounds))))
			else:
				r = list(range(self.index)) + list(range(self.index + 1, len(self.sounds)))
				self.index = choice(r)
		else:
			self.index += 1
			if self.index == len(self.sounds):
				self.index = 0

	@property
	def callback(self):
		"""Just needed as a getter for the property, not needed for the code"""
		pass

	@callback.setter
	def callback(self, cb):
		"""Sets all the sound objects to have the passed callback (cb)"""
		for s in self.sounds:
			s.callback = cb



from ..libloader.com import load_com
from .base import Output

import logging
log = logging.getLogger(__name__)

class Sapi4(Output):

 name = 'sapi4'
 priority = 102

 def __init__(self):
  sapi4 = load_com("{EEE78591-FE22-11D0-8BEF-0060081841DE}")
  self._voiceNo = sapi4.Find(0)
  sapi4.Select(self._voiceNo)
  sapi4.Speak(" ")
  self.__object = sapi4
  self._voice_list = self._available_voices()

 def _set_capabilities(self):
  sapi4 = self.__object
  try:
   sapi4.Pitch = sapi4.Pitch
   self._min_pitch = sapi4.MinPitch
   self._max_pitch = sapi4.MaxPitch
   self._has_pitch = True
  except:
   self._min_pitch = 0
   self._max_pitch = 0
   self._has_pitch = False
  try:
   sapi4.Speed = sapi4.Speed
   self._min_rate = sapi4.MinSpeed
   self._max_rate = sapi4.MaxSpeed
   self._has_rate = True
  except:
   self._min_rate = 0
   self._max_rate = 0
   self._has_rate = False
  try:
   sapi4.VolumeLeft = sapi4.VolumeLeft
   self._min_volume = sapi4.MinVolumeLeft
   self._max_volume = sapi4.MaxVolumeLeft
   self._has_volume = True
  except:
   self._min_volume = 0
   self._max_volume = 0
   self._has_volume = False

 def _available_voices(self):
  voice_list = []
  for voice_no in range(1, self.__object.CountEngines):
   voice_list.append(self.__object.ModeName(voice_no))
  return voice_list

 @property
 def available_voices(self):
  return self._voice_list

 def list_voices(self):
  return self.available_voices

 def get_voice(self):
  return self.__object.ModeName(self._voice_no)

 def set_voice(self, value):
  self._voice_no = self.list_voices().index(value) + 1
  self.__object.Select(self._voice_no)
  self.silence()
  self.__object.Speak(" ")
  self._set_capabilities()

 def get_pitch(self):
  if self.has_pitch:
   return self.__object.Pitch

 def set_pitch(self, value):
  if self.has_pitch:
   self.__object.Pitch = value

 def get_rate(self):
  if self.has_rate:
   return self.__object.Speed

 def set_rate(self, value):
  if self.has_rate:
   self.__object.Speed = value

 def get_volume(self):
  if self.has_volume:
   return self.__object.VolumeLeft

 def set_volume(self, value):
  if self.has_volume:
   self.__object.VolumeLeft = value

 @property
 def has_pitch(self):
  return self._has_pitch

 @property
 def has_rate(self):
  return self._has_rate

 @property
 def has_volume(self):
  return self._has_volume

 @property
 def min_pitch(self):
  return self._min_pitch

 @property
 def max_pitch(self):
  return self._max_pitch

 @property
 def min_rate(self):
  return self._min_rate

 @property
 def max_rate(self):
  return self._max_rate

 @property
 def min_volume(self):
  return self._min_volume

 @property
 def max_volume(self):
  return self._max_volume

 def speak(self, text, interrupt=False):
  if interrupt:
   self.silence()
  self.__object.Speak(text)

 def silence(self):
  self.__object.AudioReset()

output_class = Sapi4



from builtins import str
import os
import platform

from ..platform_utils import paths
from ..libloader import load_library
from .base import Output

class NVDA(Output):
 """Supports The NVDA screen reader"""
 name = "NVDA"
 lib32 = 'nvdaControllerClient32.dll'
 lib64 = 'nvdaControllerClient64.dll'

 def is_active(self):
  try:
   return self.lib.nvdaController_testIfRunning() == 0
  except:
   return False

 def braille(self, text, **options):
  self.lib.nvdaController_brailleMessage(str(text))

 def speak(self, text, interrupt=False):
  if interrupt:
   self.silence()
  self.lib.nvdaController_speakText(str(text))

 def silence(self):
  self.lib.nvdaController_cancelSpeech()

output_class = NVDA



import _winreg
import os
import sys
from platform_utils import paths

RUN_REGKEY = u"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run"

def is_installed(app_subkey):
 """Checks if the currently running copy is installed or portable variant. Requires the name of the application subkey found under the uninstall section in Windows registry."""

 try:
  key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\%s" % app_subkey)
  inst_dir = _winreg.QueryValueEx(key,"InstallLocation")[0]
 except WindowsError:
  return False
 _winreg.CloseKey(key)
 try:
  return os.stat(inst_dir) == os.stat(paths.app_path())
 except WindowsError:
  return False

def getAutoStart(app_name):
 """Queries if the automatic startup should be set for the application or not, depending on it's current state."""

 try:
  key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, RUN_REGKEY)
  val = _winreg.QueryValueEx(key, unicode(app_name))[0]
  return os.stat(val) == os.stat(sys.argv[0])
 except (WindowsError, OSError):
  return False

def setAutoStart(app_name, enable=True):
 """Configures automatic startup for the application, if the enable argument is set to True. If set to False, deletes the application AutoStart value."""

 if getAutoStart(app_name) == enable:
  return
 key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, RUN_REGKEY, 0, _winreg.KEY_WRITE)
 if enable:
  _winreg.SetValueEx(key, unicode(app_name), None, _winreg.REG_SZ, sys.argv[0])
 else:
  _winreg.DeleteValue(key, unicode(app_name))



#Speaks text in the way I wish without hard-coding accessible_output like normal.

# here are the dependencies for the below packages:
# libloader:
# http://hg.q-continuum.net/libloader/
# platform_utils:
# http://hg.q-continuum.net/platform_utils/

#Here is the link to the below package:
# http://hg.q-continuum.net/accessible_output2/

from .accessible_output2.outputs import auto
spk = auto.Auto().output

"""
try:
	from .accessible_output2.outputs import auto
	spk = accessible_output2.outputs.auto.Auto().output
except:
	try:
		#here is the link for the below package:
		# http://hg.q-continuum.net/accessible_output/
		from accessible_output import speech
		spk = speech.Speaker().output
	except:
		print("No speech package installed, using print")
		spk = lambda text: print(text)
"""

speechOn = True
lexicon = {' ': 'space', '\n': 'carriage return'}

def speak(text, sp=True):
	"""Is the leading function for speech"""
	if text in lexicon:
		text = lexicon.get(text, text)
	if speechOn and sp:
		spk(text)
	return Text(text)

class Text(object):
	"""Deals with processing of text"""
	def __init__(self, text):
		self.text = text
		self.word_position = 0
		self.letter_position = 0
		self.current_word = None
		self.current_letter = text[0]
		self.word_list = text.split(' ')
		for i in self.word_list:
			if i != self.word_list[-1]:
				i = " " + i

	def letter_to_word(self):
		"""Will look at the number of the letter and figure out where in the string it is and return the current word"""
		number_list = []
		n = 0
		for i in self.word_list:
			number_list.append([n, n + len(i)])
			n += len(i)
		for i in number_list:
			if self.letter_position in range(i[0], i[1]):
				self.current_word = self.word_list[number_list.index(i)]
		return self.current_word

	def word_to_letter(self):
		"""Run this and all the current letter and position will be updated"""
		f = self.text.index(self.current_word)
		self.letter_position = f
		self.current_letter = self.text[f]
		return self.current_letter

	def change_letter(self, number, speak=True):
		"""Add or subtract to this to change the current letter"""
		f = self.letter_position + number
		if f < 0:
			self.letter_position = 0
		elif f < len(self.text):
			self.letter_position = f
		else:
			self.letter_position = len(self.text)-1
		self.letter_to_word()
		self.word_position = self.word_list.index(self.current_word)
		self.current_letter = self.text[self.letter_position]
		if speak:
			self.speak(self.current_letter)
		return self.current_letter

	def change_word(self, number, speak=True):
		"""Will move word by word"""
		f = self.word_position + number
		if f < 0:
			self.word_position = 0
		elif f < len(self.word_list):
			self.word_position = f
		else:
			self.word_position = len(self.word_list)-1
		self.current_word = self.word_list[self.word_position]
		self.word_to_letter()
		if speak:
			self.speak(self.current_word)
		return self.current_word

	def navigate(self, command, speak=True):
		"""Call this with the commands: start, end, word_forward, word_back, letter_forward, letter_back"""
		text = self.text
		if command == "start":
			self.letter_position = 0
			self.word_position = 0
			self.letter_to_word()
		elif command == "end":
			self.letter_position = len(text)-1
			self.word_position = len(self.word_list)-1
			self.letter_to_word()
		elif command == "word_back":
			text = self.change_word(-1, False)
		elif command == "word_forward":
			text = self.change_word(1, False)
		elif command == "letter_back":
			text = self.change_letter(-1, False)
		elif command == "letter_forward":
			text = self.change_letter(1, False)
		if speak:
			self.speak(text)
		return text

	def speak(self, text=None):
		"""Will either speak the current text or the text that it is passed"""
		if not text:
			text = self.text
		if text in lexicon:
			text = lexicon.get(text, text)
		spk(text)

	def move(self, actions, speak=True):
		"""Will speak letters and words in a normal windows setting with right and left arrowing through the letters and ctrl+right and left arrow moving through words"""
		key = actions['key']
		mods = actions['mods']
		text = ""
		if key == "left" and not "ctrl" in mods:
			text = self.navigate("letter_back", False)
		elif key == "right" and not "ctrl" in mods:
			text = self.navigate("letter_forward", False)
		elif key == "left" and "ctrl" in mods:
			text = self.navigate("word_back", False)
		elif key == "right" and "ctrl" in mods:
			text = self.navigate("word_forward", False)
		elif key in ["up", "down"]:
			text = self.text
		elif key in "end":
			text = self.navigate("end", False)
		elif key == "home":
			text = self.navigate("start", False)
		if speak and text:
			self.speak(text)
		return text

def main():
	print(current_text)
	while True:
		c = raw_input("< direction?")
		navigate(c)

if __name__ == '__main__':
	f = Text("I like to dance")

	import pyaudiogame
	my_app = pyaudiogame.App("Letters and words")
	storage = pyaudiogame.cash
	storage.text = Text("I like to dance")
	def logic(actions):
		"""This is using the more basic change_letter and change_word functions from the word class"""
		k = actions['key']
		mods = actions['mods']
		if k == "right" and not mods:
			f.change_letter(1)
		elif k == "left" and not mods:
			f.change_letter(-1)
		elif k == "left" and "ctrl" in mods:
			f.change_word(-1)
		elif k == "right" and "ctrl" in mods:
			f.change_word(1)
		elif k == "up":
			f.speak()

	def logic(actions):
		"""This is the most simple way of using actions. You need key and mods keys in the actions dict for this to work. built for use with pyaudiogame"""
		key = actions['key']
		if key == "space":
			storage.text.speak()
		elif key:
			storage.text.move(actions)

	my_app.logic = logic
	my_app.run()



import os
from .base import Output

class AppleSay(Output):
 """Speech output supporting the Apple Say subsystem."""
 name = 'Apple Say'
 def __init__(self, voice = 'Alex', rate = '300'):
  self.voice = voice
  self.rate = rate
  super(AppleSay, self).__init__()
 def is_active(self):
  return not os.system('which say')
 def speak(self, text, interrupt = 0):
  if interrupt:
   self.silence()
  os.system('say -v %s -r %s "%s" &' % (self.voice, self.rate, text))
 def silence(self):
  os.system('killall say')

output_class = AppleSay


#For making a grid
#a Wall or wall consists of the length of the x Wall and the length of the y Wall.
#MyGrid.add_wall(0,10, 10,10)
#the first two numbers are saying that our x Wall is from 0 to 10 on the grid. that means we have a Wall that is 10 squares long.
#the second two numbers say that we wish our y Wall to go up from 10 to 10. This means that it is just 1 square wide.
import math
from pyaudiogame.utils.array_math import magnitude, vector, dotProduct, sumArray

# imports for the AdvancedGrid class
from pyaudiogame import event_queue
from pyaudiogame import KeyMap
from pyaudiogame.mixer import SoundIterator, set_listener

class Grid(object):
	"""Call this class with the width and height, then check(x, y) to see if there is something where that x,y point is."""

	def __init__(self, width=50, height=50):
		self.width = width
		self.height = height

		#object list
		self.objects = []

		self.add_wall(0, width, 0, 0)
		self.add_wall(0, width, height, height)
		self.add_wall(0, 0, 0, height)
		self.add_wall(width, width, 0, height)

	def check(self, x, y):
		"""Checks to see if the passed point is in any of the specified polys"""
		for p in self.objects:
			on_move_event = self.get_move_event(x, y, p)
			p.on_move(on_move_event)
			if on_move_event  .in_poly:
				return p.callback()

	def check_rect(self, x, y):
		"""Checks if a point is in a rect"""
		for o in self.objects:
			if x >= o.min_x and x <= o.max_x and y >= o.min_y and y <= o.max_y:
				if o.callback:
					return o.callback()
				return True

	def get_move_event(self, x, y, p):
		"""x and y are the pos of the player. p is an object with a .poly attribute."""
		in_poly = self.point_in_polygon(x, y, p.poly)
		on_move_event = OnMoveEvent((x,y), in_poly, *self.distance_and_direction_to_polygon((x,y), p.poly))
		return on_move_event  

	def point_in_polygon(self, x, y, poly):
		"""Generic point in polygon function, polygon must be a set of x y tuples in a clockwise or counterclockwise order."""
		n = len(poly)
		inside = False
		try:
			p1x, p1y = poly[0]
		except IndexError:
			return None
		for i in range(n+1):
			p2x, p2y = poly[i % n]
			if len(poly) == 2 and x >= min(p1x,p2x) and x <= max(p1x,p2x) and y >= min(p1y,p2y) and y <= max(p1y,p2y): # it is a single line
				return True
			if x == p2x and y == p2y: #it is on a point
				return True
			if y > min(p1y, p2y):
				if y <= max(p1y,p2y):
					if x < max(p1x, p2x):
						if p1y != p2y:
							xinters = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
						if p1x == p2x or x <= xinters:
#							print("ran: ", ([p1x, p1y], [p2x, p2y]))
							inside = not inside
			p1x, p1y = p2x, p2y
		return inside

	def distance_and_direction_to_polygon(self, point, poly):
		"""gets the distance and direction in angles to the polygon"""
		if self.point_in_polygon(*point, poly):
			return [0, 0, tuple(point)]
		minDist = 10000000000
		minAngle = 0
		finalNearestPoint = poly[0]
		p1 = poly[0]
		for i in range(1, len(poly)+1):
			p2 = poly[i % len(poly)]
			distance = 0
			lineVec = vector(p2, p1)
			perpVec = [lineVec[1], lineVec[0]*-1]
			dot = dotProduct(perpVec, vector(point, p1))
			distPointToLine = dot / magnitude(perpVec)
			perpVecMagnitude = magnitude(perpVec)
			normalizedPerpVec = [p/perpVecMagnitude for p in perpVec]
			dotSignedPerpVec = [p * -1 for p in normalizedPerpVec]
			intersectionPoint = sumArray([p*distPointToLine for p in dotSignedPerpVec], point)
			# get the distance between each of the points
			p1_intersectionPoint_dist = magnitude(vector(p1, intersectionPoint))
			p2_intersectionPoint_dist = magnitude(vector(p2, intersectionPoint))
			p1_p2_dist = magnitude(vector(p1, p2))
			# check if the intersectionPoint is on the p1_p2 line
			online = p1_intersectionPoint_dist + p2_intersectionPoint_dist == p1_p2_dist
			nearestPoint = intersectionPoint
			if not online:
				nearestPoint = p1 if p1_intersectionPoint_dist > p2_intersectionPoint_dist else p2
			point_to_nearest_point = vector(nearestPoint, point)
			distance = magnitude(point_to_nearest_point)
			# get the angle the point is to the line
			vecRight = [1,0]
			radAngle = math.atan2((point_to_nearest_point[0]*vecRight[1]) - (point_to_nearest_point[1]*vecRight[0]), (point_to_nearest_point[0] * vecRight[0]) + (point_to_nearest_point[1]*vecRight[1]))
			degree = radAngle * (180/math.pi)
			minDist = min(distance, minDist)
			minAngle = degree if minDist == distance else minAngle
			finalNearestPoint = nearestPoint if minDist == distance else  finalNearestPoint
			p1 = p2
		return [minDist, minAngle*-1, tuple(finalNearestPoint)]

	def add_wall(self, min_x, max_x, min_y, max_y, callback=lambda:True, on_move=lambda e:True):
		"""Adds a Wall object to the object list"""
		new_wall = Wall(min_x, max_x, min_y, max_y, callback, on_move)
		self.objects.append(new_wall)
		return new_wall

	def add_polygon(self, poly, callback=lambda:True, on_move=lambda e:True):
		"""Creates a Polygon and adds it to the objects list"""
		if type(poly) == list or type(poly) == tuple:
			p = Polygon(poly, callback, on_move)
		else:
			 p = poly
		self.objects.append(p)
		return p

class OnMoveEvent(object):
	def __init__(self, current_pos, in_poly, distance, angle, nearest_point):
		self.current_pos = current_pos
		self.in_poly = in_poly
		self.distance = distance
		self.angle = angle
		self.nearest_point = nearest_point

class Wall(object):
	"""This just has the properties for basic Wall objects"""
	def __init__(self, min_x, max_x, min_y, max_y, callback=lambda:True, on_move=lambda e:True):
		self.min_x = min_x
		self.max_x = max_x
		self.min_y = min_y
		self.max_y = max_y
		self.poly = [(min_x, min_y), (min_x, max_y), (max_x, max_y), (max_x, min_y)]
		self.callback = callback
		self.on_move = on_move
		poly_set = list(set(self.poly))
		if len(poly_set) == 2:
			self.poly = poly_set

	def __repr__(self):
		"""Call this class to see the below returned"""
		return "WallObject:x(%d, %d)y(%d, %d)" % (self.min_x, self.max_x, self.min_y, self.max_y)

class Polygon(object):
	def __init__(self, poly, callback=lambda:True, on_move=lambda e:True):
		self.poly = poly
		self.callback = callback
		self.on_move = on_move

	def __repr__(self):
		return self.poly


class AdvancedGrid(object):
	"""This is an advanced grid that will take cair of location, handling input, and playing sounds."""
	def __init__(self, width=50, height=50, step_sounds=[], hit_sounds=[], speed=1, grid=None, starting_pos=(1,1), delay=0.3, start_with_step=True, on_step=None, on_hit=None, move_listener=True, keymap=[{'key':'up', 'event':'up'},{'key':'down', 'event':'down'},{'key':'left','event':'left'},{'key':'right','event':'right'}]):
		"""arguments:
			width and height are the dimensions of the grid in an int. They both default to 50.
			speed is the amount of tiles the pos moves each check in a float
			starting_pos is the starting position of the player in a tuple.
			step_sounds are sounds that play each time the player moves. Either a list of file names can be passed in, or a SoundIterator can be passed in.
			hit_sounds are sound that play each time the player hits an impassable obstical. Either a list of file names can be passed in, or a SoundIterator can be passed in.
			keymap is a list of default keyboard mappings that will be used in the grid's local keymap. These can be modified or changed by using the keymap object at self.keymap.
			grid is a Grid object. If not passed in, then it will default to creating a new Grid object with the width and height.
			self.delay is the amount of time inbetween each step.
			start_with_step is a bool that is by default True. It means that the user will always step as soon as they press a movement key. If it is False, then there will be a loop that runs every delay and if the running variable is true, then the user will take a step. otherwise, nothing will happen. This prevents users from spamming movement keys, but is not very good for testing.
			on_hit is a callback function that runs when the user hits an object. It is passed self.
			on_step is a callback that runs when the user takes a step. It is passed self.
			move_listener is a boolian that is True by default. If true, the listener object in pyaudiogame.mixer is moved along with the point.
		"""

		self.height = height
		self.width = width
		self.speed = speed
		self.pos = starting_pos
		self.step_sounds = step_sounds if type(step_sounds) != list else SoundIterator(*step_sounds)
		self.hit_sounds = hit_sounds if type(hit_sounds) != list else SoundIterator(*hit_sounds)
		self.keymap = KeyMap(keymap)
		self.grid = grid if grid else Grid(width, height)
		self.delay = delay
		self.start_with_step = start_with_step
		self.on_hit = on_hit if on_hit else lambda s: True
		self.on_step = on_step if on_step else lambda s: True
		self.move_listener = move_listener

		# operation variables
		self.moving = False
		self._id = "AdvancedGrid%s" % id(self)
		self._current_event = None
		if not start_with_step:
			event_queue.schedule(function=self._move_callback, name=self._id, delay=self.delay, repeats=-1, before_delay=False)

	def move(self, event):
		"""This will start the player moving if they are not moving and stop the player if they are moving. Each step will be a self.delay apart."""
		state = self.keymap.getEvent(event.key, event.mods, state=1)
		if self.moving and state and event.state == 0:
			self.moving = False
			if self.start_with_step:
				event_queue.unschedule(self._id)
		elif state:
			self._current_event = event
			self.moving = True
			if self.start_with_step:
				event_queue.schedule(function=self._move_callback, name=self._id, delay=self.delay, repeats=-1, before_delay=True)

	def _move_callback(self):
		if self.moving:
			self.move_once(self._current_event)

	def move_once(self, event):
		"""Pass the event object in from an EventHandler that has events from the global_keymap and this will move the player once."""
		state = self.keymap.getEvent(key=event.key, mods=event.mods, state=event.state)
		if state:
			x, y = self.pos
			if state == "up":
				y += self.speed
			elif state == "down":
				y -= self.speed
			elif state == "right":
				x += self.speed
			elif state == "left":
				x -= self.speed
			if not self.grid.check(x, y):
				self.pos = (x, y)
				self.step_sounds.play()
				self.on_step(self)
				if self.move_listener:
					set_listener(x, y, 90)
				return False
			else:
				self.hit_sounds.play()
				self.on_hit(self)
				return True

	def add_wall(self, min_x, max_x, min_y, max_y, callback=lambda:True, on_move=lambda e:True):
		"""Adds a Wall object to the object list in the grid"""
		return self.grid.add_wall(min_x, max_x, min_y, max_y, callback, on_move)

	def add_polygon(self, poly, callback=lambda:True, on_move=lambda e:True):
		"""Creates a Polygon and adds it to the objects list"""
		return self.grid.add_polygon(poly, callback, on_move)

if __name__ == '__main__':
	my_grid = Grid(50, 50)
	o = my_grid.objects[3]
	x, y = (50, 1)
	print(o.poly)
	print(x >= o.min_x and x <= o.max_x and y >= o.min_y and y <= o.max_y)
	print(my_grid.point_in_polygon(x, y, o.poly))
	print(my_grid.point_in_polygon(x, y, [[50, 0], [50, 50]]))


"""
Manages the usage of keymaps
usage:
keymap = KeyMap()
keymap.add(key=["f"], mods=["shift"], event="fligh)
"""

class KeyMap(object):
	def __init__(self, keymap=[]):
		self._keymap = []

		self.add(keymap)

	def _addSingle(self, mapping={}, key=[], mods=[], event=None, function=None, state=1):
		"""adds a key mapping to self._keymap. One can either pass in a dict of the keywords, or use the arguments to create the mapping. If one passes in both a dict and some keywords, note that the keywords will be over-ridden by the dict. either one or both of function or event need to be passed."""
		local_mapping = locals()
		# The next section keeps mapping from adding new key, which is what it would do if one used update.
		for k in local_mapping:
			local_mapping[k] = mapping.get(k, local_mapping[k])
		del local_mapping["mapping"] # so if it is empty in the next line, it doesn't trigger as false. One could also make it equal True, but this is more explicit
		if local_mapping["key"] and (local_mapping["function"] or local_mapping["event"]):
			# make sure key and mods are sets (not lists because we check if set == set in the getEvent function and list == list doesn't work if the items are out of order)
			local_mapping["key"] = set(list(local_mapping["key"]) if type(local_mapping["key"]) == list else [local_mapping["key"]])
			local_mapping["mods"] = set(list(local_mapping["mods"]) if type(local_mapping["mods"]) == list else [local_mapping["mods"]])
			self._keymap.append(local_mapping)
		else:
			missing_items = [k for k in local_mapping if not local_mapping[k] and k != "mods"]
			missing_items = ', '.join(missing_items)
			raise Exception("KeymapError: KeyMap.add was passed a mapping without a value for: %s" % missing_items)

	def add(self, mapping={}, key=[], mods=[], event=None, function=None, state=1):
		if type(mapping) == list:
			[self._addSingle(m, key, mods, event, function, state) for m in mapping]
		else:
			self._addSingle(mapping, key, mods, event, function, state)

	def getModsSets(self, mods):
		"""converts mods into a couple different sets and returns a list. That way users can use shift,and alt, rather than leftctrl and rightctrl."""
		nutral = set()
		with_alt = set()
		nutral_with_alt = set()
		for mod in mods:
			if "meta" in mod:
				alt = mod.replace("meta", "alt")
				with_alt.add(alt)
				nutral.add("meta")
				nutral_with_alt.add("alt")
			elif not mod in ['left', 'right'] and ('left' in mod or 'right' in mod):
				with_alt.add(mod)
				mod = mod.replace('left', '').replace('right', '').strip()
				nutral.add(mod)
				nutral_with_alt.add(mod)
			else:
				with_alt.add(mod)
				nutral.add(mod)
				nutral_with_alt.add(mod)
		return [with_alt, nutral, nutral_with_alt]

	def getEvent(self, key, mods=[], state=1):
		"""Gets the event that is triggered by the list of key and mods."""
		key = set(list(key) if type(key) == list else [key])
		key = {k.lower() for k in key}
		mods = set(list(mods if type(mods) == list else [mods]))
		mods = {str(k).lower() for k in mods}
		for mapping in self._keymap:
			if mapping['key'] == key and mapping['mods'] == mods and mapping['state'] == state:
				if mapping['function']:
					return mapping['function']()
				return mapping['event']
			elif mods:
				mod_sets = self.getModsSets(mods)
				for mod_set in mod_sets:
					if mapping['key'] == key and mapping['mods'] == mod_set and mapping['state'] == state:
						if mapping['function']:
							return mapping['function']()
						return mapping['event']


if __name__ == '__main__':
	keymap = KeyMap([
		{'key':['space'], 'event': "oranges", 'state':0},
		{'key':['f', 'a'], 'mods': ['ctrl','shift'], 'function': lambda: True},
	])
	print(keymap.getEvent(key='space', state=0))


from .libloader import *

__version__ = 0.1
__author__ = 'Christopher Toth <q@q-continuum.net>'
__doc__ = """
Quickly and easily load shared libraries from various platforms. Also includes a libloader.com module for loading com modules on Windows.
"""



from pyaudiogame import global_keymap

def _run_on_callbacks(self, name):
	def returned_func(event):
		try:
			self.__dict__['_' + name](event)
		except KeyError:
			pass
		object.__getattribute__(self, name)(event)
	return returned_func

class EventTemplate(object):
	def __init__(self, event=None, input_types=['key', 'mousebutton', 'mousemove']):
		self._input_types = input_types
		self.event = event
		self.type = ''
		self.state = -1
		self.key = ''
		self._button = ''
		self.mods = []
		self.focused = None
		self.input = False
		self.mouse_x = 0
		self.mouse_y = 0
		self.mous_rel = (0, 0)

	def set_type(self, type):
		type = str(type)
		self.type = type
		if type in self._input_types:
			self.input = True

	@property
	def keymap_event(self):
		return global_keymap.getEvent(self.key, self.mods, self.state)

	@property
	def button(self):
		return self._button

	@button.setter
	def button(self, button):
		"""makes button set self.key to 'button' + button. This is so keymap and event handlers don't need to catch anything but key to get mouse clicks."""
		self._button = button
		self.key = "mouse" + str(button)

class EventHandler(object):
	def __init__(self, **kwargs):
		for k, v in kwargs.items():
			if k.startswith("on"):
				self.__dict__["_" + k] = v

# Normally the over-ridden functions when creating an EventHandler
	def convert_event(self, event):
		pass
	def run(self):
		pass
	def on_event(self, event):
		pass
	def on_input(self, event):
		pass

	def run_handlers(self, event):
		"""Requires the above EventTemplate object to be used"""
		self.on_event(event)
		if event.input:
			self.on_input(event)
		if event.type == 'quit':
			self.on_exit()
		elif event.type == 'userevent':
			self.on_user(event)
		elif event.type == 'videoexpose':
			self.on_expose(event)
		elif event.type == 'videoresize':
			self.on_resize(event)
		elif event.type == 'key' and event.state == 0:
			self.on_key_up(event)
		elif event.type == 'key' and event.state == 0:
			self.on_key_down(event)
		elif event.type == 'mousemotion':
			self.on_mouse_move(event)
		elif event.type == 'mousebutton' and event.state == 0:
			if event.button == 'left':
				self.on_lbutton_up(event)
			elif event.button == 'middle':
				self.on_mbutton_up(event)
			elif event.button == 'right':
				self.on_rbutton_up(event)
		elif event.type == 'mousebutton' and event.state == 1:
			if event.button == 'left':
				self.on_lbutton_down(event)
			elif event.button == 'middle':
				self.on_mbutton_down(event)
			elif event.button == 'right':
				self.on_rbutton_down(event)
		elif event.type == 'activeevent':
			if event.state == 'mouse':
				if event.focused:
					self.on_mouse_focus(event)
				else:
					self.on_mouse_blur(event)
			elif event.state == 'input':
				if event.focused:
					self.on_input_focus(event)
				else:
					self.on_input_blur(event)
			elif event.state == 'window':
				if event.focused:
					self.on_restore(event)
				else:
					self.on_minimize(event)

	def on_input_focus(self, event):
		pass
	def on_input_blur(self, event):
		pass
	def on_key_down(self, event):
		pass
	def on_key_up(self, event):
		pass
	def on_mouse_focus(self, event):
		pass
	def on_mouse_blur(self, event):
		pass
	def on_mouse_move(self, event):
		pass
	def on_mouse_wheel(self, event):
		pass
	def on_lbutton_up(self, event):
		pass
	def on_lbutton_down(self, event):
		pass
	def on_rbutton_up(self, event):
		pass
	def on_rbutton_down(self, event):
		pass
	def on_mbutton_up(self, event):
		pass
	def on_mbutton_down(self, event):
		pass
	def on_minimize(self, event):
		pass
	def on_restore(self, event):
		pass
	def on_resize(self, event):
		pass
	def on_expose(self, event):
		pass
	def on_exit(self, event):
		pass
	def on_user(self, event):
		pass
	def on_joy_axis(self,event):
		pass
	def on_joybutton_up(self,event):
		pass
	def on_joybutton_down(self,event):
		pass
	def on_joy_hat(self,event):
		pass
	def on_joy_ball(self,event):
		pass
	def on_event(self, event):
		pass

	def __getattribute__(self, name):
		"""This runs a check to see if there are any internal functions to call before the handler runs. This is so the engine can set its own internal attributes and still allow users to create their own handlers"""
		if name.startswith("on"):
			return _run_on_callbacks(self, name)
		return object.__getattribute__(self, name)



# Replacement for py2exe distributed module
# Avoids the use of the standard py2exe console.
# Just import this file and it should go away

import sys
if hasattr(sys,"frozen"): # true only if we are running as a py2exe app
 class Blackhole(object):
  def write(self,text):
   pass
  def flush(self):
   pass
 sys.stdout = Blackhole()
 sys.stderr = Blackhole()
 del Blackhole
 del sys




#Use this module to store variables and settings
# The variables that can't be stored here are:
# _pickle, save, and load.
# usage:
"""
import storage
storage.hitpoints = 100
storage.hitpoints -= 4
storage.save()
# then on reload of the app
storage.load()
print(storage.hitpoints)
>>> 96
"""
try:
	import cPickle as _pickle
except:
	import pickle as _pickle

def save(file="save.sav", level=1):
	"""Call this function with the location of the file where you wish to save the info in this module. If the level == 0, it will just return a dict of everything that would be saved. If level == 1, it will save to a pickled file."""
	g = globals()
	storage_dict = {}
	[storage_dict.update({i: g[i]}) for i in g if not i.startswith("_")]
	del(storage_dict['save'])
	del(storage_dict['load'])
	if level == 1:
		with open(file, 'wb') as f:
			_pickle.dump(storage_dict, f, protocol=_pickle.HIGHEST_PROTOCOL)
	return storage_dict

def load(file="save.sav", level=1):
	"""This will return the dict of all the variables in the storage module. If there is no file, it will return an empty dict. If level == 0, it will just return a dict of all the items that are in the save file. If level == 1, it will overwrite any variables in storage that have the same variable as the saved variables. If level == 2, the storage variables will not be overwritten, but new variables will be added from the save."""
	try:
		with open(file, "rb") as f:
			loaded_file = _pickle.load(f)
	except:
		return {}
	if level == 1:
		[globals().update({i: loaded_file[i]}) for i in loaded_file]
	elif level == 2:
		g = globals()
		[globals().update({i: loaded_file[i]}) for i in loaded_file if i not in g]
	return loaded_file



# These are simple equasions dealing with arrays. It is to avoid using numpy. If this file becomes too complex though, numpy will be needed.
import math

def magnitude(v):
	return math.sqrt(v[0]**2 + v[1]**2)

def vector(p1, p2):
	return [p1[0] - p2[0], p1[1] - p2[1]]


def dotProduct(v1, v2):
	return ((v1[0] * v2[0]) + (v1[1] * v2[1]))

def sumArray(a1,a2):
	return [a1[i]+a2[i] for i in range(len(a1))]



import _winreg

SHELL_REGKEY = u"Directory\\shell"

def context_menu_integrate(item_key_name, item_display_text, item_command):
 app_menu_key = _winreg.OpenKey(_winreg.HKEY_CLASSES_ROOT, SHELL_REGKEY, 0, _winreg.KEY_WRITE)
 menu_item_key = _winreg.CreateKey(app_menu_key, item_key_name)
 _winreg.SetValueEx(menu_item_key, None, None, _winreg.REG_SZ, item_display_text)
 item_command_key = _winreg.CreateKey(menu_item_key, 'command')
 _winreg.SetValueEx(item_command_key, None, None, _winreg.REG_SZ, item_command)



from __future__ import absolute_import
import platform
if platform.system() == 'Windows':
 from . import nvda
 from . import jaws
 from . import sapi5
 from . import window_eyes
 from . import system_access
 from . import dolphin
 from . import pc_talker
 #from . import sapi4

if platform.system() == 'Darwin':
 from . import voiceover
 from . import say

if platform.system() == 'Linux':
 from . import e_speak

from .import auto



import inspect
import platform
import os
import sys

from functools import wraps

def merge_paths(func):
 @wraps(func)
 def merge_paths_wrapper(*a, **k):
  return unicode(os.path.join(func(**k), *a))
 return merge_paths_wrapper

def windows_path(path_id):
 import ctypes
 path = ctypes.create_unicode_buffer(260)
 if ctypes.windll.shell32.SHGetFolderPathW(0, path_id, 0, 0, ctypes.byref(path)) != 0:
 	raise ctypes.WinError()
 return path.value

def app_data_path(app_name=None):
 """Cross-platform method for determining where to put application data."""
 """Requires the name of the application"""
 plat = platform.system()
 if plat == 'Windows':
  path = windows_path(0x01a)
 elif plat == 'Darwin':
  path = os.path.join(os.path.expanduser('~'), 'Library', 'Application Support')
 elif plat == 'Linux':
  path = os.path.expanduser('~')
  app_name = '.%s' % app_name.replace(' ', '_')
 return os.path.join(path, app_name)

def prepare_app_data_path(app_name):
 """Creates the application's data directory, given its name."""
 dir = app_data_path(app_name)
 if not os.path.exists(dir):
  os.mkdir(dir)

def is_frozen():
 """Return a bool indicating if application is compressed"""
 import imp
 return hasattr(sys, 'frozen') or imp.is_frozen("__main__")

def get_executable():
 if is_frozen():
  return sys.executable
 return sys.argv[0]

def get_module():
 return inspect.stack()[2][1]

def app_path():
 """Always determine the path to the main module, even when run with py2exe or otherwise frozen"""
 return os.path.abspath(os.path.dirname(get_executable()))

def module_path():
 return os.path.abspath(os.path.dirname(get_module()))

def executable_path():
 return os.path.join(app_path(), get_executable())

def ensure_path(path):
  if not os.path.exists(path):
   os.makedirs(path)

def documents_path():
 """On windows, returns the path to My Documents. On OSX, returns the user's Documents folder. For anything else, returns the user's home directory."""
 plat = platform.system()
 if plat == 'Windows':
  return windows_path(0x005)
 elif plat == 'Darwin':
  path = os.path.join(os.path.expanduser('~'), 'Documents')
 else:
  path = os.path.expanduser('~')
 return path



from __future__ import absolute_import
import ctypes
import os
import types
from .platform_utils import paths

name = 'accessible_output2'

def load_library(libname, cdll=False):
 if paths.is_frozen():
  libfile = os.path.join(paths.embedded_data_path(), 'accessible_output2', 'lib', libname)
 else:
  libfile = os.path.join(paths.module_path(), 'lib', libname)
 if cdll:
  return ctypes.cdll[libfile]
 else:
  return ctypes.windll[libfile]

def get_output_classes():
 from . import outputs
 module_type = types.ModuleType
 classes = [m.output_class for m in outputs.__dict__.values() if type(m) == module_type and hasattr(m, 'output_class')]
 return sorted(classes, key=lambda c: c.priority)

def find_datafiles():
 import os
 import platform
 from glob import glob
 import accessible_output2
 if platform.system() != 'Windows':
  return []
 path = os.path.join(accessible_output2.__path__[0], 'lib', '*.dll')
 results = glob(path)
 dest_dir = os.path.join('accessible_output2', 'lib')
 return [(dest_dir, results)]



import unittest
from pyaudiogame.keymap import KeyMap

def funcCallback():
	return True

class TestKeymap(unittest.TestCase):
	def test_add_function(self):
		keymap = KeyMap()
		keymap.add(key=["f"], mods=["shift"], event="fligh")
		keymap.add({
			"key": "j",
			"event": "jump"
		})
		self.assertEqual(keymap.getEvent(key=["f"], mods=["shift"]), "fligh")
		self.assertEqual(keymap.getEvent(key="j"), "jump")
		self.assertEqual(keymap.getEvent("space"), None)

	def test_add_in_object(self):
		keymap = KeyMap([
			{'key':['space'], 'event': "oranges"},
			{'key':['f', 'a'], 'mods': ['ctrl','shift'], 'function': funcCallback},
		])
		self.assertEqual(keymap.getEvent(key=['f', 'a'], mods=['ctrl','shift']), funcCallback())
		self.assertEqual(keymap.getEvent(key='space'), 'oranges')

	def test_add_a_list(self):
		keymap = KeyMap()
		keymap.add([
			{'key':'space', 'event': "oranges"},
			{'key':['f', 'a'], 'mods': ['ctrl','shift'], 'function': funcCallback},
		])
		self.assertEqual(keymap.getEvent(key=['f', 'a'], mods=['ctrl','shift']), funcCallback())
		self.assertEqual(keymap.getEvent(key='space'), 'oranges')

	def test_state(self):
		keymap = KeyMap([
			{'key':'space', 'mods':['shift'], 'state':0, 'event':'shiftSpace'},
			{'key':'a', 'state':1, 'event':'apple'},
		])
		self.assertEqual(keymap.getEvent(key='space', mods='shift', state=0), 'shiftSpace')
		self.assertEqual(keymap.getEvent('a', state=0), None)

	def test_nutral_mods(self):
		keymap = KeyMap([
			{'key': 'f', 'mods': ['alt', 'ctrl', 'shift'], 'event':'modded'},
		])
		self.assertEqual(keymap.getEvent(key="f", mods=['alt', 'ctrl', 'shift']), 'modded')
		self.assertEqual(keymap.getEvent(key="f", mods=['leftalt', 'rightctrl', 'shift']), 'modded')
		self.assertEqual(keymap.getEvent(key="f", mods=['leftmeta', 'rightctrl', 'leftshift']), 'modded')

	def test_quit_event(self):
		keymap = KeyMap([
			{'key': 'escape', 'event': 'quit'},
			{'key': 'f4', 'mods':['alt'], 'event': 'quit2'}
		])
		self.assertEqual(keymap.getEvent(key='escape'), 'quit')
		self.assertEqual(keymap.getEvent(key='F4', mods=['left alt']), 'quit2')



if __name__ == '__main__':
	unittest.main


"""Simple example showing how to get keyboard events.
Note that the mouse events don't work very well. Something is wrong with the pipe process that keeps the mouse event process from exiting in the inputs script. The bug has been reported, and as soon as it is fixed, uncommenting the run_mouse() function will work.
print(event.ev_type, event.code, event.state)
"""
import sys

from .inputs import get_key, get_mouse

from pyaudiogame.event_handler import EventHandler, EventTemplate

mod_keys = [
'capslock',
'left shift',
'right shift',
'left ctrl',
'right ctrl',
'f15', 
'right meta',
'left meta'
]

event_names = {
	'esc': 'escape',
	'enter': 'return',
	'f15': 'help',
	'102nd': '\\',
	'minus': '-',
	'apostrophe': "`",
	'equal': '=',
	'backslash': "'",
	'semicolon': ';',
	'rightbrace': ']',
	'leftbrace': '[',
}

duplicated_keys = [
	'backspace',
	'delete',
	'up',
	'down',
	'right',
	'left',
	'page up',
	'page down',
	'home',
	'end',
]

mouse_types = {
	'Absolute': 'mousemotion',
	'Key': 'mousebutton',
}

class Console(EventHandler):
	def __init__(self, **kwargs):
		EventHandler.__init__(self, **kwargs)
		self.mod_keys = mod_keys
		self.pressed = set()
		self.activated_already = set()
		self.last_mouse_coords = (0, 0)
		self.event_queue = []
		# these below lists lines are queues for each type of event type because the get_event functions are blocking.
		self.mouse_events = []
		self.keyboard_events = []

	def run(self):
		self.event_queue = []
		self.run_key(self.event_queue)
#		self.run_mouse(self.event_queue)
		[self.run_handlers(e) for e in self.event_queue if e.input]

	def processCode(self, code):
		"""makes the key code the same as pygame's"""
		code = code.split("_")[-1].lower()
		mod_code = code.replace('left', '').replace('right', '')
		if 'page' in code:
			code = 'page ' + code[4:]
		if mod_code in ['ctrl', 'shift', 'meta']:
			code = code.replace('left', 'left ').replace('right', 'right ')
		code = event_names.get(code, code)
		return code

	def run_mouse(self, queue):
		events = get_mouse(False)
		coords = [0,0]
		mousemotion_event = None
		for event in events:
			event_type, code, state = event.ev_type, event.code.lower(), event.state
			event_obj = EventTemplate(event)
			event.mods = self.pressed
			if '_x' in code:
				coords[0] = state
				mousemotion_event = event
				continue
			elif '_y' in code:
				coords[1] = state
				continue
			elif 'btn_' in code:
				event_obj.set_type('mousebutton')
				event_obj.button = code[4:]
				event_obj.state = state
				if event.state == 1:
					self.pressed.add(event)
				elif event in self.pressed:
					self.pressed.remove(event)
			else:
				event_obj.set_type(event_type)
				event_obj.state = state
			queue.append(event_obj)
		if coords[0] or coords[1]:
			event_obj = EventTemplate(mousemotion_event)
			event_obj.set_type('mousemotion')
			event.mods = self.pressed
			event_obj.mouse_x, event_obj.mouse_y = coords
			event_obj.mouse_rel = (self.last_mouse_coords[0] - coords[0], self.last_mouse_coords[1] - coords[1])
			self.last_mouse_coords = coords
			queue.append(event_obj)

	def run_key(self, queue):
		events = get_key(False)
		for event in events:
			event = self.convert_key_event(event)
			if not event: # this is to make sure the duplicated events don't run
				continue
			queue.append(event)
			if event.state == 1 and event.input and not event.key in [e.key for e in self.pressed]:
				self.pressed.add(event)

	def convert_key_event(self, event): 
		ev_type, code, state = [event.ev_type, event.code, event.state]
		code = self.processCode(code)
		# make sure no duplicate keys are pressed
		if code in duplicated_keys and (code, state) in self.activated_already:
			self.activated_already.remove((code, state))
			return False
		else:
			self.activated_already.add((code, state))
		in_mods = [e for e in self.pressed if code == e.key]
		if not state and in_mods:
			self.pressed.remove(in_mods[0])
		event_obj = EventTemplate()
		event_obj.event = event
		event_obj.set_type(ev_type.lower())
		event_obj.state = state
		event_obj.mods = list(self.pressed)
		if event_obj.type == 'key':
			event_obj.key = code
		return event_obj

if __name__ == "__main__":
	def main():
		"""Just print out some event infomation when keys are pressed."""
		def run_func(event):
			if event.type == 'key':
				print(event.key)
				print("mods are: %s" % event.mods)

			elif event.type == "mousebutton" and event.state:
				print("clicked!")

		console = Console(run_func)
		while 1:
			console.run()

	main()


#Tests the ui.grid module
import unittest
from pyaudiogame.ui import grid
my_grid = grid.Grid(50, 50)

class TestGrid(unittest.TestCase):
	def test_wall(self):
		dumb_wall = grid.Wall(1,10, 0,1)
		self.assertEqual(dumb_wall.max_y, 1)
		self.assertEqual(dumb_wall.min_x, 1)
		self.assertEqual(dumb_wall.poly, [(1,0), (1,1), (10, 1), (10, 0)])

	def test_sides(self):
		self.assertEqual(my_grid.check_rect(50, 1), True)
		self.assertEqual(my_grid.check(50, 1), True)
		self.assertEqual(my_grid.check_rect(5,50), True)
		self.assertEqual(my_grid.check(5,50), True)
		self.assertEqual(my_grid.check_rect(5,5), None)
		self.assertEqual(my_grid.check(5,5), None)
		#This is because the sides of the whole thing are just 1 square wide.
		self.assertEqual(my_grid.check_rect(51,1), None)
		self.assertEqual(my_grid.check(51,1), None)

	def test_adding_areas(self):
		my_grid.add_wall(4,6, 3,3)
		my_grid.add_wall(4,6, 6,6)
		my_grid.add_wall(4,4, 3,6)
		my_grid.add_wall(6,6, 3,4)
		#check inside the box
		self.assertEqual(my_grid.check_rect(5,5), None)
		self.assertEqual(my_grid.check(5,5), None)
		#test the wall
		self.assertEqual(my_grid.check_rect(5,6), True)
		self.assertEqual(my_grid.check(5,6), True)
		#test the door
		self.assertEqual(my_grid.check_rect(6,5), None)
		self.assertEqual(my_grid.check(6,5), None)

	def test_callback_function(self):
		"""Test that you can make walls that you can walk through and the callback is run"""
		my_grid.add_wall(7,9, 2,5, lambda:None)
		my_grid.add_wall(20,25, 2,5, lambda:"pizza")
		self.assertEqual(my_grid.check_rect(7,3), None)
		self.assertEqual(my_grid.check(7,3), None)
		self.assertEqual(my_grid.check_rect(22,4), "pizza")
		self.assertEqual(my_grid.check(22,4), "pizza")

	def test_point_in_polygon(self):
		point_in_polygon = grid.Grid.point_in_polygon
		poly_square = [
			[2,10], [10,10],
			[10,5], [2, 5],
		]
		# inside the polygon
		self.assertEqual(point_in_polygon(None, 4, 6, poly_square), True)
		# test the edges
		self.assertEqual(point_in_polygon(None, 2, 6, poly_square), True)
		# Check the corner
		self.assertEqual(point_in_polygon(None, 2, 5, poly_square), True)
		# Check a wall in the grid
		# tests the wall poly
		wall = grid.Wall(0,10, 0,1)
		self.assertEqual(point_in_polygon(None, 4, 1, wall.poly), True)
		# test a line
		line = [[50, 0], [50, 50]]
		self.assertEqual(point_in_polygon(None, 50, 1, line), True)

	def test_create_poly(self):
		points = [(2,2), (2, 10), (12, 10), (12, 2)]
		my_grid = grid.Grid(50, 50)
		my_grid.add_polygon(points)
		self.assertEqual(my_grid.check(4,4), True)

	def test_distance_and_direction_to_polygon(self):
		poly = [(1,3),(1,4),(2,4), (2,3)]
		point1 = [-2,1]
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point1, poly)[1], 45)
		self.assertAlmostEqual(my_grid.distance_and_direction_to_polygon(point1, poly)[0], 4.24, places=2)
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point1, poly)[2], (1,4))
		point2 = [1,1, ]
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point2, poly), [2.0, 90.0, (1.0, 3.0)])
		point3 = [1, 5]
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point3, poly), [1.0, -90.0, (1.0, 4.0)])
		point4 = [3, 3]
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point4, poly), [1.0, 180.0, (2.0, 3.0)])
		point5 = [-3, 3]
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point5, poly), [4.0, 0, (1.0, 3.0)])
		point6 = [1,3]
		self.assertEqual(my_grid.distance_and_direction_to_polygon(point6, poly), [0, 0, (1.0, 3.0)])



#This is a module that creates menus.
#you call the menus.Menu with all the settings, then you cicle using the Menu.run(key, mod) call.
"""
my notes:
if using a numbered list look at:
for i, v in enumerate(['tic', 'tac', 'toe']):
	print i, v

"""
from pyaudiogame.speech import speak as spk

class Menu(object):
	"""Used for creating a menu in your program"""

	def __init__(self, options=["yes", "no"], default_position=0, keys={"back": "up", "forward": "down", "exit": ["escape", "backspace"], "except": "return"}, shortkeys=None, title="Test Menu", loops=True, persistent=0, sounds={}):
		"""This creates a menu with the following values:
			options: are what the user sees displayed. If you pass a list, all items will have the settings of font. if you pass a dict, the keys will be the options and the values will be font settings for each line, like the font argument.
			default_position: the first selected option
			keys: pass a string or a list of strings for each entry. back is if you were going up in the menu, forward is down in the menu, exit is to leave, except is to choose an option.
			shortkeys: Create a list of key names that is as long as the options list and pass it. For example ['1', '2', '3'] will activate options 0, 1 and 2 in the options list.
			title: The text that is said before the menu. pass as None if you don't wish any.
			loops: when using the keyboard, says if you will reach the bottom of the screen and do nothing or if you will loop back to the start. also vice-versa
			persistent: will remember your position. has 4 options, 0 doesn't remember anything, 1 remembers only on exit, 2 only remembers on selection, 3 remembers on both
			sounds: are sounds for different actions. the excepted values are: "change": for when the selection changes, "select": for when the selection is clicked on, "exit": when the menu is closed, "hit": when
				using the keyboard, will make a sound when hitting the top or bottom and loop is not on, "pass": for when loop is on and using the keyboard, will make this sound when switching between the
				top and bottom of the menu. It is best to use mp3 or ogg and needs pygame to run.

			Note, if you just run this menu with the defaults, it will be a yes and no menu. All you need to run it is:
			menu1 = menus.Menu(title="Who would you like to date?", options=["fred", "frank", "Julia", "sally"])
		"""

		self.options = options
		self.default_position = default_position
		self.keys = keys
		self.shortkeys = shortkeys
		self.title = title
		self.loops = loops
		self.persistent = persistent
		self.sounds = sounds

		#These are class spacific variables
		self.current_position = default_position
		self.say_title = True
		self.message = None
		self.ran = False

		#default checks
		default_key_dict = {"back": "up", "forward": "down", "exit": ["escape", "backspace"], "except": "return"}
		[i.update({i: default_key_dict[i]}) for i in default_key_dict if not self.keys.get(i)]

	def run(self, actions):
		"""Is the function you call every time you run the menu. Just pass in a dict of actions"""
		actions['key'] = str(actions['key'])
		self.message = None
		key = actions['key']
		keys = self.keys
		result = None
		if not self.ran:
			self.first_run()

		if key in keys["back"] or key in keys['forward']:
			self.key_checker(key)
		elif key in keys['exit']:
			result = self.exit_menu()
		elif key in keys['except']:
			result = self.chose_item()

		if self.message:
			spk(self.message)
		return result

	def first_run(self):
		if self.say_title and self.title:
			spk(self.title)
			self.message = self.options[self.current_position]
			self.say_title = False
			self.ran = False

	def key_checker(self, key):
		"""checks what key it is and does stuff accordingly"""
		if key == self.keys['back']:
			self.change_position("add")
		elif key == self.keys['forward']:
			self.change_position('minus')
		self.message = self.options[self.current_position]

	def exit_menu(self):
		"""Resets the menu for the next call"""
		self.say_title = True
		self.options_display = []
		if self.persistent in [0, 2]:
			self.current_position = self.default_position
		return "exit"

	def chose_item(self):
		"""resets the menu and returns the current option"""
		self.say_title = True
		self.options_display = []
		position = self.current_position
		if self.persistent in [0, 1]:
			self.current_position = self.default_position
		return self.options[position]

	def change_position(self, direction):
		p = self.current_position
		o = len(self.options) - 1
		if direction == "minus":
			p += 1
		elif direction == "add":
			p -= 1
		if p > o:
			if self.loops:
				p = 0
			else:
				p -= 1
		elif p < 0:
			if self.loops:
				p = o
			else:
				p += 1
		self.current_position = p

def add_menu(actions, options=['Yes', 'No'], dict={}, name=None, result_list=[], default_position=0, keys={"back": "up", "forward": "down", "exit": ["escape", "backspace"], "except": "return"}, shortkeys=None, title="Test Menu", loops=True, persistent=0, sounds={}):
	"""Call this function with a dict of actions and the name of a dict to append either the name or title to."""
	if not name:name = title
	if not dict.get(name):
		dict[name] = Menu(options, default_position, keys, shortkeys, title, loops, persistent, sounds)
	else:
		s = dict[name].run(actions)
		if result_list and s:
			if s == "exit":
				result_list.pop()
			else:
				result_list.append(s)
		return s



#Player template

class Player(object):
	"""There are some basic functions and variables that a player may need"""
	def __init__(self, name="Fred", location=(1, 1)):
		self.name = name
		self.location = location

		#settings
		self.map = None

	def move(self, actions, map=None):
		"""call this to move the player"""
		if not map:
			map = self.map
		x = self.location[0]
		y = self.location[1]
		key = actions['key']
		if key == "up":
			y += 1
		elif key == "down":
			y -= 1
		elif key == "right":
			x += 1
		elif key == "left":
			x -= 1
		if not map.check(x, y):
			self.location = (x, y)
			return False
		return True




import pygame
from pygame.locals import *
from pyaudiogame.event_handler import EventHandler, EventTemplate

mod_id = {
64: ['left ctrl'],
320: ['left ctrl'],
1: ['left shift'],
512: ['right alt'],
578: ['left ctrl', 'right shift', 'right alt'],
513: ['right alt', 'left shift'],
514: ['right alt', 'right shift'],
257: ['left shift'],
256: ['left alt'],
2: ['right shift'],
128: ['right ctrl'],
65: ['left ctrl', 'left shift'],
66: ['left ctrl', 'right shift'],
257: ['left alt', 'left shift'],
129: ['right ctrl', 'left shift'],
130: ['right ctrl', 'right shift'],
321: ['left ctrl', 'left alt', 'left shift'],
322: ['left ctrl', 'left alt', 'right shift'],
258: ['left alt', 'right shift'],
384: ['left alt', 'right ctrl'],
386: ['left alt', 'right ctrl', 'right shift'],
}

event_types = {
	KEYDOWN: 'key',
	KEYUP: 'key',
	MOUSEBUTTONDOWN: 'mousebutton',
	MOUSEBUTTONUP: 'mousebutton',
	ACTIVEEVENT: 'activeevent',
	USEREVENT: 'userevent',
	VIDEOEXPOSE: 'videoexpose',
	VIDEORESIZE: 'videoresize',
	MOUSEMOTION: 'mousemotion',
}

activeevent_states = {
	1: 'mouse',
	2: 'input',
	4: 'window'
}

class PygameInput(EventHandler):
	def __init__(self, **kwargs):
		EventHandler.__init__(self, **kwargs)

	def run(self):
		for event in pygame.event.get():
			event = self.convert_event(event)
			self.run_handlers(event)

	def convert_event(self, event):
		event_dict = event.__dict__
		event_obj = EventTemplate()
		event_obj.event = event
		event_obj.set_type(event_types.get(event.type, event.type))
		mods = mod_id.get(pygame.key.get_mods(), [pygame.key.get_mods()])
		if mods != [0]:
			event_obj.mods = mods
		if event.type in [KEYDOWN, MOUSEBUTTONDOWN]:
			event_obj.state = 1
		elif event.type in [KEYUP, MOUSEBUTTONUP]:
			event_obj.state = 0
		if event_dict.get('key'):
			event_obj.key = pygame.key.name(event.key).lower()
		elif event_obj.type == 'mousebutton':
			if event.button == 0:
				event_obj.button = 'left'
			elif event.button == 1:
				event_obj.button = 'middle'
			elif event.button == 2:
				event_obj.button = 'right'
			elif event.button == 4:
				event_obj.button = 'rollup'
			elif event.button == 5:
				event_obj.button = 'rolldown'
		elif event_obj.type == 'activeevent':
			event_obj.state = activeevent_states.get(event.state, event.state)
		elif event_obj.type == 'mousemotion':
			event_obj.mouse_x, event_obj.mouse_y = pygame.mouse.get_pos()
			event_obj.mouse_rel = pygame.mouse.get_rel()
		if event_dict.get('gain') != None:
			event_obj.focused = event.gain
		return event_obj



"""
This Scheduler is quite easy. There are only two calls you need to know about, tick and schedule.
Here is an example:

import time
#Our little framerate (better to use something a little more advanced, but this works just fine.
fps = 0.03

def hello(name):
	#Just a random little function that prints 'hello name'
	print("hello %s" % name)

#There is one argument that goes in Scheduler, it is the amount you wished each time to be multiplied by. Pygame for example, ticks in incraments of 1000 each second where as time.sleep is in incraments of 1 for 1 second.
#Default is 1.0, for pygame it would be 0.001.

c = Scheduler()
c.schedule(function=hello, delay=1, repeats=2, before_delay=True, name="Fred")

while c.events:
	c.tick(fps)
	time.sleep(fps)
"""

class Scheduler:
	"""Call tick to run a tick and schedular to add an event to a queue"""

	def __init__(self, time_format=1):
		self.events = set()
		self.new_events = set()
		self.time_format = time_format


	def tick(self, elapsed_time):
		"""Call this each iteration of the game loop with an argument of the amount of time that has elapsed in seconds"""
		done_events = []
		for event in self.events:
			if event.should_run():
				event.run()
			if event.done:
				done_events.append(event)
			event.tick(elapsed_time*self.time_format)
		self.events = self.events | self.new_events
		self.new_events = set()
		[self.events.remove(e) for e in done_events]

	def schedule(self, function, delay=0, repeats=1, before_delay=False, name=None, *args, **kwargs):
		"""function is the name of the callback function that will run with the given arguments, delay is the amount of time to wait (0 for every tick), repeats is the amount of times the event will run (0 or less) for infinent, before_delay says that the function run before the delay, name is the title of the event, and the wrest are arguments for the function"""
		e = EventMaker(function, delay, repeats, before_delay, name, *args, **kwargs)
		self.new_events.add(e)

	def unschedule(self, event_name):
		"""Call this with the event name as a string to remove it from the queue"""
		events = self.events | self.new_events
		for i in events:
			if i.name == event_name:
				i.done = True

class EventMaker:
	"""This class is the event. It has all the functions to run the event."""
	def __init__(self, function, delay, repeats, before_delay, name, *args, **kwargs):
		"""function is the name of the callback function that will run with the given arguments, delay is the amount of time to wait (0 for every tick), repeats is the amount of times the event will run (0 or less) for infinent, before_delay says that the function run before the delay, name is the title of the event, and the wrest are arguments for the function"""
		self.function = function
		self.delay = delay
		self.repeats = repeats
		self.before_delay = before_delay
		self.name = name
		self.args = args
		self.kwargs = kwargs

		#Our operation variables:
		self.elapsed_time = 0
		self.done = False

	def tick(self, elapsed_time):
		"""adds time to the elapsed_time"""
		self.elapsed_time += elapsed_time

	def should_run(self):
		"""Checks if the event should run"""
		if self.before_delay and not self.elapsed_time and self.repeats:
			self.repeats += 1
			return True
		elif self.elapsed_time >= self.delay:
			return True

	def run(self):
		"""Runs the event"""
		self.repeats -= 1
		if self.repeats or not self.before_delay:
			self.function(*self.args, **self.kwargs)
		if self.repeats == 0:
			self.done = True
		self.elapsed_time = 0


if __name__ == '__main__':
	def hello(name):
		print("hello %s" % name)

	c = Scheduler(0.001)
	c.schedule(hello, 1, 2, True, name="Fred")

	import pygame
	pygame.init()
	fps = 30
	fpsClock = pygame.time.Clock()

	while c.events:
		c.tick(fpsClock.tick(fps))


# Everything is run off pygame ticks here, so 1000 is one second
from pyaudiogame.ticker import Scheduler
event_queue = Scheduler(time_format=0.001)

def tick_seconds(seconds):
	"""Ticks the specified number of seconds"""
	fps = 30
	number_of_frames = int((seconds*1000)/fps)
	for i in range(number_of_frames):
		event_queue.tick(0.03)




from .base import Output

try:
 import espeak.core
except:
 raise RuntimeError("Cannot find espeak.core. Please install python-espeak")

class ESpeak(Output):
 """Speech output supporting ESpeak on Linux
    Note this requires python-espeak to be installed
    This can be done on Debian distros by using apt-get install python-espeak
    Or through this tarball: https://launchpad.net/python-espeak
 """
 name = "Linux ESpeak"

 def is_active(self):
  try:
   import espeak.core
  except:
   return False
  return True

 def speak(self, text, interrupt = 0):
  if interrupt:
   self.silence()
  espeak.core.synth(text)
 def silence(self):
  espeak.core.cancel()

output_class = ESpeak



import win32gui
from ..libloader.com import load_com
from .base import Output, OutputError
import pywintypes

class WindowEyes (Output):
 """Speech output supporting the WindowEyes screen reader"""

 name = 'Window-Eyes'

 def __init__(self, *args, **kwargs):
  super(WindowEyes, self).__init__(*args, **kwargs)
  try:
   self.object = load_com("gwspeak.speak")
  except pywintypes.com_error:
   raise OutputError

 def speak(self, text, interrupt=0):
  if interrupt:
   self.silence()
  self.object.SpeakString(text)

 def silence (self):
  self.object.Silence()

 def is_active(self):
  try:
   return win32gui.FindWindow("GWMExternalControl", "External Control") != 0
  except:
   return False

output_class = WindowEyes 



from builtins import str
import os

from .base import Output

class Dolphin (Output):
 """Supports dolphin products."""

 name = 'Dolphin'
 lib32 = 'dolapi.dll'

 def speak(self, text, interrupt=0):
  if interrupt:
   self.silence()
  #If we don't call this, the API won't let us speak.
  if self.is_active():
   self.lib.DolAccess_Command(str(text), (len(text)*2)+2, 1)

 def silence(self):
  self.lib.DolAccess_Action(141)

 def is_active(self):
  try:
   return self.lib.DolAccess_GetSystem() in (1, 4, 8)
  except:
   return False

output_class = Dolphin



from .. import load_library
import platform

class OutputError(Exception):
 pass

class Output(object):
 name = "Unnamed Output"
 lib32 = None
 lib64 = None
 cdll = False
 priority = 100
 system_output = False

 def __init__(self):
  self.is_32bit = platform.architecture()[0] == "32bit"
  if self.lib32 and self.is_32bit:
   self.lib = load_library(self.lib32, cdll=self.cdll)
  elif self.lib64:
   self.lib = load_library(self.lib64, cdll=self.cdll)
  else:
   self.lib = None

 def output(self, text, **options):
  output = False
  if self.speak(text, **options):
   output = True
  if self.braille(text, **options):
   output = True
  if not output:
   raise RuntimeError("Output %r does not have any method defined to output" % self)

 def is_system_output(self):
  return self.system_output

 def speak(self, **optiont):
  return False

 def braille(self, **options):
  return False



import subprocess, os

from base import Output



class VoiceOver(Output):

 """Speech output supporting the Apple VoiceOver screen reader."""

 def runAppleScript(self, command, process = 'voiceover'):

  return subprocess.Popen(['osascript', '-e', 'tell application "' + process + '"\n' + command + '\nend tell'], stdout = subprocess.PIPE).communicate()[0]

 name = 'VoiceOver'

 def speak(self, text, interrupt=0):

  if interrupt:

   self.silence()

  os.system('osascript -e \"tell application \\\"voiceover\\\" to output \\\"%s\\\"\" &' % text)

 def silence (self):

  self.runAppleScript('output ""')

 def is_active(self):

  return self.runAppleScript('return (name of processes) contains "VoiceOver"', 'system events').startswith('true') and not self.runAppleScript('try\nreturn bounds of vo cursor\non error\nreturn false\nend try').startswith('false')



output_class = VoiceOver



"""
The same mixer deals with sound (loaded before playing) and music (loaded while playing).
In Python, a module is actually imported the first time only. The next times the module is only linked.
The module should import pygame and eventually initialize it in the beginning of the module, or in a function called once. The main initialization can happen in the beginning of the main module. On the other hand the pygame.init() function can be called several times during the initialization phase without a problem. Just make sure that pre_init is called before any pygame.init() or the parameter will be ignored.
Turning the module into a reusable one is not that easy. I have tried to make a components-based framework engine in 2013 but I'm not sure if it would be good enough. A simpler module mimicking pySonic API might be better. It would manage the sound sources and move them when the listener is moving.
The stereo() function already deals with sounds behind the listener: the volume is divided by 2.
Here is a documented usable module from the engine framework, called pos.py, which essentially contains the formulas:

#start module
This module can be combined with pygame mixer to have a portable
positional audio system. The formulae are purely empiric. They have
been taken from SoundMUD and SoundRTS.

example:
import pos

pos.set_listener(10, 10, 90)
print pos.stereo(20, 10)
"""

import math


_x, _y, _o = 0, 0, 0

def set_listener(x, y, o):
	"""sets the position of the listener:

	- x, y: coordinates of the listener
	- o: orientation in degrees, east is 0 degrees, counterclockwise
	"""
	global _x, _y, _o
	_x, _y, _o = x, y, o

def _distance(x, y):
	"""distance of x, y from the listener"""
	dx = x - _x
	dy = y - _y
	return math.sqrt(dx * dx + dy * dy)

def _angle(x, y):
	"""angle of x, y observed from the listener"""
	d = _distance(x, y)
	if d == 0:
		return 0 # object too close => in front of the listener
	ac = math.acos((x - _x) / d)
	if y - _y > 0:
		a = ac
	else:
		a = -ac
	return a - math.radians(_o)

def stereo(x, y, nodist=False, above=False):
	"""returns the left and right volumes

	- x, y: coordinates of the object
	- nodist: distance = 1 (for signaling object direction)
	- above: from above
	"""
	# TODO: above
	a = _angle(x, y)
	d = _distance(x, y)
	left = (math.sin(a) + 1) / 2.0
	right = 1 - left
	left = math.sin(left * math.pi / 2.0)
	right = math.sin(right * math.pi / 2.0)
	if math.cos(a) < 0: # behind
		if nodist:
			k = 1.3
		else:
			k = 2.0  # TODO: attenuate less? (especially in overhead view)
		left /= k
		right /= k
	if d < 1 or nodist:
		d = 1
	left = min(left / d, 1)
	right = min(right / d, 1)
	return left, right

def get_listener():
	"""returns the current position of the listener"""
	return (_x, _y, _o)

if __name__ == "__main__":
	assert stereo(0, 0)
	assert set_listener(10,10,90) == None
	assert get_listener() == (10, 10, 90)



from ... import accessible_output2
from .base import Output, OutputError

class Auto(Output):

 def __init__(self):
  output_classes = accessible_output2.get_output_classes()
  self.outputs = []
  for output in output_classes:
   try:
    self.outputs.append(output())
   except OutputError:
    pass

 def get_first_available_output(self):
  for output in self.outputs:
   if output.is_active():
    return output
  return None

 def speak(self, *args, **kwargs):
  output = self.get_first_available_output()
  if output:
   output.speak(*args, **kwargs)

 def braille(self, *args, **kwargs):
  output = self.get_first_available_output()
  if output:
   output.braille(*args, **kwargs)

 def output(self, *args, **kwargs):
  output = self.get_first_available_output()
  if output:
   output.speak(*args, **kwargs)

 def is_system_output(self):
  output = self.get_first_available_output()
  if output:
   return output.is_system_output()



from collections import OrderedDict
from ..libloader.com import load_com
from .base import Output, OutputError
import pywintypes
import logging
log = logging.getLogger(__name__)

SVSFDefault = 0
SVSFlagsAsync = 1
SVSFPurgeBeforeSpeak = 2
SVSFIsFilename = 4
SVSFIsXML = 8
SVSFIsNotXML = 16
SVSFPersistXML = 32

class SAPI5(Output):
 has_volume = True
 has_rate = True
 has_pitch = True
 min_pitch = -10
 max_pitch = 10
 min_rate = -10
 max_rate = 10
 min_volume = 0
 max_volume = 100
 name = "sapi5"
 priority = 101
 system_output = True

 def __init__(self):
  try:
   self.object = load_com("SAPI.SPVoice")
   self._voices = self._available_voices()
  except pywintypes.com_error:
   raise OutputError
  self._pitch = 0

 def _available_voices(self):
  _voices = OrderedDict()
  for v in self.object.GetVoices():
   _voices[v.GetDescription()] = v
  return _voices

 def list_voices(self):
  return self._voices.keys()

 def get_voice(self):
  return self.object.Voice.GetDescription()

 def set_voice(self, value):
  log.debug("Setting SAPI5 voice to \"%s\"" % value)
  self.object.Voice = self._voices[value]
  # For some reason SAPI5 does not reset audio after changing the voice
  # By setting the audio device after changing voices seems to fix this
  # This was noted from information at:
  # http://lists.nvaccess.org/pipermail/nvda-dev/2011-November/022464.html
  self.object.AudioOutput = self.object.AudioOutput

 def get_pitch(self):
  return self._pitch

 def set_pitch(self, value):
  log.debug("Setting pitch to %d" % value)
  self._pitch = value

 def get_rate(self):
  return self.object.Rate

 def set_rate(self, value):
  log.debug("Setting rate to %d" % value)
  self.object.Rate = value

 def get_volume(self):
  return self.object.Volume

 def set_volume(self, value):
  self.object.Volume = value

 def speak(self, text, interrupt=False):
  if interrupt:
   self.silence()
  # We need to do the pitch in XML here
  textOutput = "<pitch absmiddle=\"%d\">%s</pitch>" % (round(self._pitch), text.replace("<", "&lt;"))
  self.object.Speak(textOutput, SVSFlagsAsync | SVSFIsXML)
 
 def silence(self):
  self.object.Speak("", SVSFlagsAsync | SVSFPurgeBeforeSpeak)

 def is_active(self):
  if self.object:
   return True
  return False

output_class = SAPI5



import ctypes
from .base import Output

class PCTalker(Output):
 lib32 = 'pctkusr.dll'
 lib64 = 'pctkusr64.dll'
 cdll = True

 def speak(self, text, interrupt=False):
  if interrupt:
   self.silence()
  self.lib.PCTKPRead(text.encode('cp932', 'replace'), 0, 1)

 def silence(self):
  self.lib.PCTKVReset()

 def is_active(self):
  return self.lib.PCTKStatus() != 0

output_class = PCTalker



import win32gui
from ..libloader.com import load_com
import pywintypes

from .base import Output, OutputError

class Jaws (Output):
 """Output supporting the Jaws for Windows screen reader."""

 name = 'jaws'

 def __init__(self, *args, **kwargs):
  super (Jaws, self).__init__(*args, **kwargs)
  try:
   self.object = load_com("FreedomSci.JawsApi", "jfwapi")
  except pywintypes.com_error:
   raise OutputError

 def braille(self, text, **options):
  # HACK: replace " with ', Jaws doesn't seem to understand escaping them with \
  text = text.replace('"', "'")
  self.object.RunFunction("BrailleString(\"%s\")" % text)

 def speak(self, text, interrupt=False):
  self.object.SayString('      %s' % text, interrupt)

 def is_active(self):
  try:
   return self.object.SayString('',0) == True or win32gui.FindWindow("JFWUI2", "JAWS") != 0
  except:
   return False

output_class = Jaws



from pywintypes import com_error
from win32com.client import gencache

def prepare_gencache():
 gencache.is_readonly = False
 gencache.GetGeneratePath()



def load_com(*names):
 result = None
 for name in names:
  try:
   result = gencache.EnsureDispatch(name)
   break
  except com_error:
   continue
 if result is None:
  raise com_error("Unable to load any of the provided com objects.")
 return result




import ctypes
import collections
import platform
import os

TYPES = {
 'Linux': {
  'loader': ctypes.CDLL,
  'functype': ctypes.CFUNCTYPE,
  'prefix': 'lib',
  'extension': '.so'
 },
 'Darwin': {
  'loader': ctypes.CDLL,
  'functype': ctypes.CFUNCTYPE,
  'prefix': 'lib',
  'extension': '.dylib'
 },
}
if platform.system() == 'Windows':
 TYPES['Windows'] = {
  'loader': ctypes.WinDLL,
  'functype': ctypes.WINFUNCTYPE,
  'prefix': "",
  'extension': '.dll'
 }

class LibraryLoadError(OSError): pass

def load_library(library, x86_path='.', x64_path='.', *args, **kwargs):
 lib = find_library_path(library, x86_path=x86_path, x64_path=x64_path)
 loaded = _do_load(lib, *args, **kwargs)
 if loaded is not None:
  return loaded
 raise LibraryLoadError('unable to load %r. Provided library path: %r' % (library, path))

def _do_load(file, *args, **kwargs):
 loader = TYPES[platform.system()]['loader'] 
 return loader(file, *args, **kwargs)

def find_library_path(libname, x86_path='.', x64_path='.'):
 libname = '%s%s' % (TYPES[platform.system()]['prefix'], libname)
 if platform.architecture()[0] == '64bit':
  path = os.path.join(x64_path, libname)
 else:
  path = os.path.join(x86_path, libname)
 ext = get_library_extension()
 path = '%s%s' % (path, ext)
 return os.path.abspath(path)

  
def get_functype():
 return TYPES[platform.system()]['functype']

def get_library_extension():
 return TYPES[platform.system()]['extension']



from setuptools import setup, find_packages
#from setuptools.command.develop import develop
#from setuptools.command.install import install
#from subprocess import check_call

#class PostDevelopCommand(develop):
#	"""Post-installation for development mode."""
#	def run(self):
#		check_call("pip install pywin32")
#		develop.run(self)

#class PostInstallCommand(install):
#	"""Post-installation for installation mode."""
#	def run(self):
#		check_call("pip install pywin32")
#		install.run(self)

config = {
	'description': 'PyAudioGame is a toolkit for making audio games in python.',
	'author': 'Brandon Keith Biggs',
	'url': 'https://github.com/frastlin/PyAudioGame',
	'download_url': 'https://codeload.github.com/frastlin/PyAudioGame/zip/master',
	'author_email': 'brandonkeithbiggs@gmail.com',
	'version': '0.0.2',
	'license': 'MIT',
	'packages': find_packages(),
	'package_data': {'pyaudiogame': ['accessible_output2/lib/*']},
	"zip_safe": False,
	'scripts': [],
	'name': 'pyaudiogame',
#	'cmdclass': {
#		'develop': PostDevelopCommand,
#		'install': PostInstallCommand,
#	},
	'install_requires': ['pygame'],
}

setup(include_package_data=True, **config)


## coding: latin-1
#This module is used to display and handle typing of text.
from pyaudiogame.speech import speak as spk

class Typer(object):

	#This is a dict that has default character sets"""
	character_sets = {
		'numbers': ['1','2','3','4','5','6','7','8','9','0'],
		'letters_simple': ['a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z'],
		'capitals': ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','S','Y','Z'],
		'letters_special': False,
		'punctuation_simple': ['.',',','_','\"','\''],
		'punctuation_medium': ['\\','/','[',']','`','=','-'],
		'punctuation_complex': False
	}

	#This is a dict with all the modification keys
	mod_id = {
	64: ['ctrl'],
	320: ['ctrl'],
	1: ['shift'],
	257: ['shift'],
	65: ['ctrl', 'shift'],
	256: ['alt'],
	257: ['alt', 'shift'],
	321: ['ctrl', 'alt', 'shift']
	}

	#This is a dict with all the shifted keys in it.
	shifters = {'space':'space', '-':'_', '\'':'\"', '=':'+', ';':':', '[':'{', ']':'}', '.':'>', ',':'<', '/':'?', '\\':'|', '`':'~',

	'1':'!', '2':'@', '3':'#', '4':'$', '5':'\%', '6':'^', '7':'&', '8':'*', '9':'(', '0':')'
	}

	def __init__(self, title="", valid_characters=['letters_simple', 'capitals', 'numbers', 'punctuation_simple'], multiline=False, length=0, current_string=""):
		#This creates a list of all the valid chars we can have typed. if it is in the set list, that set list is appended, otherwise, the string directaly is appended.
		self.title = title
		self.valid_characters = self.valid_list_maker(valid_characters)
		self.multiline = multiline
		#The next line will update character_sets with our boolian values for the complex stuff.
		[self.character_sets.update({i: True}) for i in valid_characters if isinstance(self.character_sets.get(i), bool)]
		self.length = length
		self.current_string = current_string

		#These are our operation variables:
		self.ran = False

	def run(self, actions):
		"""Run this to catch input and to return a string."""
		key = actions['key']
		if "shift" in actions['mods']:
			key = self.shifter(key)
		text = None
		if not self.ran:
			text = self.first_run()

		if key in self.valid_characters:
			self.current_string += key
			text = key
		elif key == "space":
			self.current_string += " "
			text = "space"
		elif key == "backspace" and self.current_string:
			text = self.current_string[-1]
			self.current_string = self.current_string[:-1]
		elif key in ['up', 'down']:
			text = self.title + ' ' + self.current_string
		elif key == 'return' and not self.multiline:
			self.ran = False
			return self.current_string


		if text:
			if text in self.character_sets['capitals']:
				text = "capital %s" % text
			spk(text)
		return None


	def first_run(self):
		"""Is the creation of the first string of text"""
		self.ran = True
		return self.title + self.current_string

	def valid_list_maker(self, options):
		"""Will check if the option is in the character_sets and if so will add that set to the list. if not, it will just add the option to the returned list."""
		char_list = []
		for o in options:
			check = self.character_sets.get(o)
			if isinstance(check, (list, tuple, set)):
				char_list += check
			else:
				char_list += o
		return char_list

	def shifter(self, key):
		"""Changes characters from their unshifted state to their shifted state"""
		k = self.shifters.get(key)
		if not k:
			k = key.upper()
		return k

def add_typer(actions, title="", default_text="", dict={}, name=None, character_sets=[]):
	"""You pass the name of the thing you wish to have text for and it is either added to the dict or ran"""
	if not name:
		name = title
	if dict.get(name):
		t = dict[name].run(actions)
	else:
		if character_sets:
			dict.update({name: Typer(title=title, current_string=default_text, valid_characters=character_sets)})
		else:
			dict.update({name: Typer(title=title, current_string=default_text)})
		t = None
	return t



#Has functions to do everything your engine module needs
import pygame, random, sys
from pygame.locals import *

from pyaudiogame.speech import speak as spk
from pyaudiogame.pygame_input import PygameInput
from pyaudiogame.console import Console
from pyaudiogame import global_keymap

#our program spacific modules:
import pyaudiogame.ticker as ticker

#This is a global event queue
event_queue = ticker.Scheduler(time_format=0.001)

class App(object):
	"""This is to subclass for an application."""

	def __init__(self, title="Test app", fps=30, window_type="pygame"):
		"""Title is the name of the window, fps is frames per second, how many iterations the loop runs in a given second. The default is 30"""
		self.title = title
		self.fps = fps
		self.window_type = window_type # pygame, console. are the options

		#Default variables that can be changed, but are not called
		#For the screen:
		#The two below numbers are in pixels, the current screen is a little box on the screen
		self.windowwidth = 640
		self.windowheight = 480
		#Full screen expands everything and fills the whole screen
		self.fullscreen = False
		#If the mouse should be visible or not
		self.mouse = False
		#Our event queue
		self.event_queue = event_queue
		# The input objects
		self.pygame_events = PygameInput(on_input=self._on_input)
		self.console_events = Console(on_input=self._on_input)

		#Our exicution variables
		self.running = True

		#This function runs to set default variables if people don't wish to mess with super
		self.set_defaults()

	def logic(self, actions):
		"""This is a function that is a placeholder. A dict of the event object is passed in and this is run in the on_input handler."""
		pass

	def in_main_loop(self):
		"""A function that is run every game loop that can be over ridden"""
		pass

	def set_defaults(self):
		"""Call this function to change the default variables"""
		pass

	def run(self):
		"""Call this when you are ready to start your game. It will run your main loop and create your screen"""
		#Call the screen
		self.create_surface(windowwidth=self.windowwidth, windowheight=self.windowheight, title=self.title, fullscreen=self.fullscreen, mouse=self.mouse)
		# add the event handlers specified in self
		self._add_handlers()
		#the clock for checking that we run 30 times a second:
		fpsClock = pygame.time.Clock().tick
		fps = self.fps
		# in_main_loop is a placeholder function that is run every loop
		in_main_loop = self.in_main_loop
		#tick is for events that are scheduled
		tick = event_queue.tick
		#create the game loop
		while True:
			self.pygame_events.run()
			if self.window_type == 'console':
				self.console_events.run()
			if not self.running:
				break
			in_main_loop()
			tick(fpsClock(fps))
		self.quit()

	def create_surface(self, windowwidth=640, windowheight=480, title="Test Surface", fullscreen=False, mouse=True):
		"""will make the main window"""
		pygame.init()
		if fullscreen:
				displaySurface = pygame.display.set_mode((windowwidth, windowheight), pygame.FULLSCREEN)
		else:
				displaySurface = pygame.display.set_mode((windowwidth, windowheight))
		pygame.display.set_caption(title)
		if self.window_type == "console":
			pygame.display.iconify() #makes the console go in front and the pygame window hide behind
			pygame.display.set_caption("Don't focus this window, Go back to the console!")
		if not mouse:
			pygame.mouse.set_visible(0)
			return displaySurface

	def _on_input(self, input_event):
		"""Handles the logic function and exiting"""
		event = input_event.keymap_event
		if event == "quit" or self.logic(input_event.__dict__) == False:
			self.running = False

	def old__on_input(self):
		"""Will return a dict of all the keyboard and other input events of pygame's event system"""
		exit_key = self.exit_key
		#a dict with all the commands that go on
		actions = {
			'mousex': 0,
			'mousey': 0,
			'key': None,
			'mouseClicked': False,
			'mods': [],
			'state': None,
			'keyUp': None
			}
		for event in pygame.event.get():
			if event.type == MOUSEMOTION:
				actions['mousex'], actions['mousey'] = event.pos
			elif event.type == MOUSEBUTTONUP:
				actions['mousex'], actions['mousey'] = event.pos
				actions['mouseClicked'] = True
			elif event.type == KEYDOWN:
				actions['state'] = "down"
				actions['key'] = pygame.key.name(event.key)
				actions['mods'] = mod_id.get(pygame.key.get_mods(), [pygame.key.get_mods()])
			elif event.type == KEYUP:
				actions['state'] = "up"
				actions['keyUp'] = pygame.key.name(event.key)
				actions['mods'] = mod_id.get(pygame.key.get_mods(), [])
			if event.type == QUIT or (actions['key'] == "escape" and exit_key == 1) or ('alt' in actions['mods'] and actions['key'] == 'f4' and exit_key == 2):
				return False
		return actions

	def add_handler(self, handler_func, name=None, window=None):
		"""adds an event handler to the event handlers. Window can be either "console", "pygame", or None. If None, it will do both. The name is to specify the handler name, such as on_input if the name is different than the handler_func's name."""
		if not name:
			name = handler_func.__name__
		if window == "console":
			self.console_events.__dict__[name] = handler_func
		elif window == "pygame":
			self.pygame_events.__dict__[name] = handler_func
		else:
			self.pygame_events.__dict__[name] = handler_func
			self.console_events.__dict__[name] = handler_func

	def _add_handlers(self):
		"""Adds all the on_ functions in self to the two event handlers."""
		d = self.__dict__.items()
		for k, v in d:
			if k.startswith("on_"):
				self.add_handler(v, name=k)

	def key_repeat(self, on=True, delay=1000, delay_before_first_repeat=1):
		"""Call this function to either start what happens when a key is held down or to turn it off."""
		if on:
			pygame.key.set_repeat(delay_before_first_repeat, delay)
		else:
			pygame.key.set_repeat()

	def quit(self):
		"""Runs the functions needed to quit gracefully from pygame and python"""
		pygame.quit()
		sys.exit()

if __name__ == '__main__':
	f = App("Key Test", window_type="console")
	p = []
	def logic(actions):
		global p
		if(actions['key']):
#			spk(actions['key'])
			if not actions['key'] in p and actions['state']:
#				print(actions['key'])
				print('%s with mods %s' % (actions['key'], actions['mods']))
				p.append(actions['key'])
			elif not actions['state'] and actions['key'] in p:
				print("released: %s" % actions['key'])
				p.remove(actions['key'])
#		mods = actions['mods']
#		if mods:
#			spk(str(mods))

	f.logic = logic
	f.run()


from builtins import str
from .base import Output

class SystemAccess (Output):
 """Supports System Access and System Access Mobile"""

 name = "System Access"
 lib32 = 'saapi32.dll'
 priority = 99

 def braille(self, text, **options):
  self.lib.SA_BrlShowTextW(str(text))

 def speak(self, text, interrupt=False):
  if self.is_active():
   self.dll.SA_SayW(str(text))

 def is_active(self):
  try:
   return self.dll.SA_IsRunning()
  except:
   return False

output_class = SystemAccess



from pyaudiogame.keymap import KeyMap

# The default keymap. If there is no quit mapping, then you will need to go to the command prompt and hit ctrl+c to exit the window.
# the key mapping object
global_keymap = KeyMap([
	{'key': 'f4', 'mods': ['alt'], 'event': 'quit'},
	{'key': 'escape', 'event':'quit'}
])

from pyaudiogame.app import App
from pyaudiogame.app import event_queue
from pyaudiogame.speech import speak
import pyaudiogame.storage